@@ -44,7 +44,8 @@ def _check_status(url, r, expected):
     """
     if expected == r.status_code:
         return
-    elif 400 == r.status_code:
+    print(r)
+    if 400 == r.status_code:
         raise FatalError('Application at "' + url  + '" can not process this request as it is bad', r.status_code, r.text)
     elif 401 == r.status_code:
         raise FatalError('Not authorized to execute commands for Application at "' + url, r.status_code, r.text)
@@ -85,25 +86,57 @@ class ItemsFunction:
             self.debug_separator()
 
 
+    def _prepare_query(self):
+        return None
+
+
     def report(self, action):
-        document = self._prepare_document()
         action_uri = action.find('uri').text
-        self.__pretty_print(action_uri, ET.tostring(document), False)
-        r = self.session.get(action_uri, data=ET.tostring(document), headers=HEADERS)
-        _check_status(action_uri, r, 200)
+        query = self._prepare_query()
+        if None == query:
+            uri_to_use = action_uri
+        else:
+            uri_to_use = action_uri + '?' + query
+        attach = action.find('attachment')
+        if None != attach:
+            media_type = attach.text
+            document = self._prepare_document(media_type)
+            self._pretty_print(uri_to_use, ET.tostring(document), False)
+            r = self.session.get(uri_to_use, data=ET.tostring(document), headers=HEADERS)
+        else:
+            r = self.session.get(uri_to_use, headers=HEADERS)
+        _check_status(uri_to_use, r, 200)
         report = ET.fromstring(r.text)
-        self.__pretty_print(action_uri, ET.tostring(report))
-        Display.items_result(report, 'Results of ' + action.find('name').text)
+        self._pretty_print(action_uri, ET.tostring(report))
+        self._display_report(action,
+                             report)
+
+
+    def _prepare_document(self, media_type):
+        """
+        Prepares a Bundles document contains the specified bundle identities
+        """
+        raise FatalError('Can not handle "' + media_type + '" requested for this action"', -5, None)
 
 
     def command(self, action):
         if None == self.session:
             return
-        document = self._prepare_document()
         action_uri = action.find('uri').text
-        self._pretty_print(action_uri, ET.tostring(document), False)
-        r = self.session.post(action_uri, data=ET.tostring(document), headers=HEADERS)
-        _check_status(action_uri, r, 202)
+        attach = action.find('attachment')
+        if None != attach:
+            media_type = attach.text
+            document = self._prepare_document(media_type)
+            self._pretty_print(action_uri, ET.tostring(document), False)
+            r = self.session.post(action_uri, data=ET.tostring(document), headers=HEADERS)
+        else:
+            r = self.session.post(action_uri, headers=HEADERS)
+        try:
+            _check_status(action_uri, r, 200)
+        except FatalError as e:
+            if 202 != e.code:
+                raise e
         report = ET.fromstring(r.text)
         self._pretty_print(action_uri, ET.tostring(report))
-        Display.items_result(report, 'Results of ' + action.find('name').text)
+        self._display_report(action,
+                             report)
